# HG changeset patch
# Parent 3b3adad9a7318dc2cc4118c88cada607fa04c6e2

diff -r 3b3adad9a731 sys/dev/cxgbe/t4_main.c
--- a/sys/dev/cxgbe/t4_main.c	Thu Jul 31 22:08:32 2014 -0700
+++ b/sys/dev/cxgbe/t4_main.c	Thu Jul 31 22:18:16 2014 -0700
@@ -30,6 +30,7 @@
 
 #include "opt_inet.h"
 #include "opt_inet6.h"
+#include "opt_kdtrace.h"
 
 #include <sys/param.h>
 #include <sys/conf.h>
@@ -46,6 +47,7 @@
 #include <dev/pci/pci_private.h>
 #include <sys/firmware.h>
 #include <sys/sbuf.h>
+#include <sys/sdt.h>
 #include <sys/smp.h>
 #include <sys/socket.h>
 #include <sys/sockio.h>
@@ -67,6 +69,8 @@
 #include "t4_ioctl.h"
 #include "t4_l2t.h"
 
+SDT_PROVIDER_DEFINE(if_cxgbe);
+
 /* T4 bus driver interface */
 static int t4_probe(device_t);
 static int t4_attach(device_t);
diff -r 3b3adad9a731 sys/dev/cxgbe/t4_sge.c
--- a/sys/dev/cxgbe/t4_sge.c	Thu Jul 31 22:08:32 2014 -0700
+++ b/sys/dev/cxgbe/t4_sge.c	Thu Jul 31 22:18:16 2014 -0700
@@ -30,6 +30,7 @@
 
 #include "opt_inet.h"
 #include "opt_inet6.h"
+#include "opt_kdtrace.h"
 
 #include <sys/types.h>
 #include <sys/eventhandler.h>
@@ -40,6 +41,7 @@
 #include <sys/malloc.h>
 #include <sys/queue.h>
 #include <sys/sbuf.h>
+#include <sys/sdt.h>
 #include <sys/taskqueue.h>
 #include <sys/time.h>
 #include <sys/sysctl.h>
@@ -69,6 +71,8 @@
 #include "common/t4_regs_values.h"
 #include "common/t4_msg.h"
 
+SDT_PROVIDER_DECLARE(if_cxgbe);
+
 #ifdef T4_PKT_TIMESTAMP
 #define RX_COPY_THRESHOLD (MINCLSIZE - 8)
 #else
@@ -1871,6 +1875,36 @@ t4_wrq_tx_locked(struct adapter *sc, str
 	}
 }
 
+/*
+ * if_cxgbe:tx::frame (information about every frame transmitted by the NIC)
+ * arg0 = txq
+ * arg1 = # of GL entries (0 for immediate data)
+ * arg2 = payload length
+ */
+SDT_PROBE_DEFINE3(if_cxgbe, tx,, frame, frame, "struct sge_txq *", "u_int",
+    "u_int");
+
+/*
+ * if_cxgbe:tx::work-request (information about every NIC tx work request)
+ * arg0 = txq
+ * arg1 = type (0 = txpkt, 1 = txpkts)
+ * arg2 = number of frames sent in the work request (always 1 for type=0)
+ * arg3 = number of descriptors used by the work request
+ * arg4 = payload carried for transmission (whether imm. or GL)
+ */
+SDT_PROBE_DEFINE5(if_cxgbe, tx,, work_request, work-request, "struct sge_txq *",
+    "u_int", "u_int", "u_int", "u_int");
+
+/*
+ * if_cxgbe:tx::doorbell (information about every NIC txq doorbell)
+ * arg0 = txq
+ * arg1 = number of descriptors handed off to the chip.
+ * XXX: # of WRs and # of frames would be nice too.
+ * XXX: will probably move to within ring_eq_db in the future (for db type etc.)
+ */
+SDT_PROBE_DEFINE2(if_cxgbe, tx,, doorbell, doorbell, "struct sge_txq *",
+    "u_int");
+
 /* Per-packet header in a coalesced tx WR, before the SGL starts (in flits) */
 #define TXPKTS_PKT_HDR ((\
     sizeof(struct ulp_txpkt) + \
@@ -1971,6 +2005,9 @@ t4_eth_tx(struct ifnet *ifp, struct sge_
 			continue;
 		}
 
+		SDT_PROBE3(if_cxgbe, tx,, frame, txq, sgl.nsegs,
+		    m->m_pkthdr.len);
+
 		if (coalescing &&
 		    add_to_txpkts(pi, txq, &txpkts, m, &sgl) == 0) {
 
@@ -2016,8 +2053,10 @@ t4_eth_tx(struct ifnet *ifp, struct sge_
 		if (sgl.nsegs == 0)
 			m_freem(m);
 doorbell:
-		if (eq->pending >= 8)
+		if (eq->pending >= 8) {
+			SDT_PROBE2(if_cxgbe, tx,, doorbell, txq, eq->pending);
 			ring_eq_db(sc, eq);
+		}
 
 		can_reclaim = reclaimable(eq);
 		if (can_reclaim >= 32)
@@ -2049,8 +2088,10 @@ doorbell:
 	}
 	txq->m = m;
 
-	if (eq->pending)
+	if (eq->pending) {
+		SDT_PROBE2(if_cxgbe, tx,, doorbell, txq, eq->pending);
 		ring_eq_db(sc, eq);
+	}
 
 	reclaim_tx_descs(txq, 0, 128);
 
@@ -3636,6 +3677,8 @@ write_txpkt_wr(struct port_info *pi, str
 	if (ndesc > eq->avail)
 		return (ENOMEM);
 
+	SDT_PROBE5(if_cxgbe, tx,, work_request, txq, 0, 1, ndesc, pktlen);
+
 	/* Firmware work request header */
 	wr = (void *)&eq->desc[eq->pidx];
 	wr->op_immdlen = htobe32(V_FW_WR_OP(FW_ETH_TX_PKT_WR) |
@@ -3874,6 +3917,9 @@ write_txpkts_wr(struct sge_txq *txq, str
 
 	ndesc = howmany(txpkts->nflits, 8);
 
+	SDT_PROBE5(if_cxgbe, tx,, work_request, txq, 1, txpkts->npkt, ndesc,
+	    txpkts->plen);
+
 	wr = (void *)&eq->desc[eq->pidx];
 	wr->op_pkd = htobe32(V_FW_WR_OP(FW_ETH_TX_PKTS_WR));
 	ctrl = V_FW_WR_LEN16(howmany(txpkts->nflits, 2));
diff -r 3b3adad9a731 sys/dev/cxgbe/tom/t4_cpl_io.c
--- a/sys/dev/cxgbe/tom/t4_cpl_io.c	Thu Jul 31 22:08:32 2014 -0700
+++ b/sys/dev/cxgbe/tom/t4_cpl_io.c	Thu Jul 31 22:18:16 2014 -0700
@@ -29,6 +29,8 @@
 __FBSDID("$FreeBSD$");
 
 #include "opt_inet.h"
+#include "opt_inet6.h"
+#include "opt_kdtrace.h"
 
 #ifdef TCP_OFFLOAD
 #include <sys/param.h>
@@ -38,6 +40,7 @@
 #include <sys/module.h>
 #include <sys/protosw.h>
 #include <sys/domain.h>
+#include <sys/sdt.h>
 #include <sys/socket.h>
 #include <sys/socketvar.h>
 #include <sys/sglist.h>
@@ -57,6 +60,8 @@
 #include "tom/t4_tom_l2t.h"
 #include "tom/t4_tom.h"
 
+SDT_PROVIDER_DECLARE(t4_tom);
+
 VNET_DECLARE(int, tcp_do_autosndbuf);
 #define V_tcp_do_autosndbuf VNET(tcp_do_autosndbuf)
 VNET_DECLARE(int, tcp_autosndbuf_inc);
@@ -1428,6 +1433,8 @@ do_abort_rpl(struct sge_iq *iq, const st
 	return (0);
 }
 
+SDT_PROBE_DEFINE4(t4_tom, ddp,, indicate, indicate, "u_int", "struct socket *",
+    "uint32_t", "u_int");
 static int
 do_rx_data(struct sge_iq *iq, const struct rss_header *rss, struct mbuf *m)
 {
@@ -1563,11 +1570,8 @@ do_rx_data(struct sge_iq *iq, const stru
 			 * the socket buffer as usual.
 			 */
 
-#if 0
-			CTR5(KTR_CXGBE,
-			    "%s: tid %u (0x%x) DDP indicate (seq 0x%x, len %d)",
-			    __func__, tid, toep->flags, be32toh(cpl->seq), len);
-#endif
+			SDT_PROBE4(t4_tom, ddp,, indicate, tid, so,
+			    be32toh(cpl->seq), len);
 			sb->sb_flags |= SB_DDP_INDICATE;
 		} else if ((toep->ddp_flags & (DDP_OK|DDP_SC_REQ)) == DDP_OK &&
 		    tp->rcv_wnd > DDP_RSVD_WIN && len >= sc->tt.ddp_thres) {
diff -r 3b3adad9a731 sys/dev/cxgbe/tom/t4_ddp.c
--- a/sys/dev/cxgbe/tom/t4_ddp.c	Thu Jul 31 22:08:32 2014 -0700
+++ b/sys/dev/cxgbe/tom/t4_ddp.c	Thu Jul 31 22:18:16 2014 -0700
@@ -29,6 +29,8 @@
 __FBSDID("$FreeBSD$");
 
 #include "opt_inet.h"
+#include "opt_inet6.h"
+#include "opt_kdtrace.h"
 
 #include <sys/param.h>
 #include <sys/types.h>
@@ -39,6 +41,7 @@
 #include <sys/protosw.h>
 #include <sys/proc.h>
 #include <sys/domain.h>
+#include <sys/sdt.h>
 #include <sys/socket.h>
 #include <sys/socketvar.h>
 #include <sys/uio.h>
@@ -65,6 +68,8 @@
 #include "common/t4_tcb.h"
 #include "tom/t4_tom.h"
 
+SDT_PROVIDER_DECLARE(t4_tom);
+
 #define PPOD_SZ(n)	((n) * sizeof(struct pagepod))
 #define PPOD_SIZE	(PPOD_SZ(1))
 
@@ -885,6 +890,10 @@ unwire_ddp_buffer(struct ddp_buffer *db)
 	}
 }
 
+SDT_PROBE_DEFINE4(t4_tom, ddp,, post, post, "u_int", "struct socket *",
+    "int", "struct ddp_buffer *");
+SDT_PROBE_DEFINE2(t4_tom, ddp,, off, off, "u_int", "struct socket *");
+
 static int
 handle_ddp(struct socket *so, struct uio *uio, int flags, int error)
 {
@@ -900,13 +909,6 @@ handle_ddp(struct socket *so, struct uio
 
 	SOCKBUF_LOCK_ASSERT(sb);
 
-#if 0
-	if (sb->sb_cc + sc->tt.ddp_thres > uio->uio_resid) {
-		CTR4(KTR_CXGBE, "%s: sb_cc %d, threshold %d, resid %d",
-		    __func__, sb->sb_cc, sc->tt.ddp_thres, uio->uio_resid);
-	}
-#endif
-
 	/* XXX: too eager to disable DDP, could handle NBIO better than this. */
 	if (sb->sb_cc >= uio->uio_resid || uio->uio_resid < sc->tt.ddp_thres ||
 	    uio->uio_resid > MAX_DDP_BUFFER_SIZE || uio->uio_iovcnt > 1 ||
@@ -963,6 +965,7 @@ handle_ddp(struct socket *so, struct uio
 	t4_wrq_tx(sc, wr);
 	sb->sb_flags &= ~SB_DDP_INDICATE;
 	toep->ddp_flags |= buf_flag;
+	SDT_PROBE4(t4_tom, ddp,, post, toep->tid, so, db_idx, db);
 
 	/*
 	 * Wait for the DDP operation to complete and then unwire the pages.
@@ -977,6 +980,7 @@ handle_ddp(struct socket *so, struct uio
 	unwire_ddp_buffer(db);
 	return (rc);
 no_ddp:
+	SDT_PROBE2(t4_tom, ddp,, off, toep->tid, so);
 	disable_ddp(sc, toep);
 	discourage_ddp(toep);
 	sb->sb_flags &= ~SB_DDP_INDICATE;
diff -r 3b3adad9a731 sys/dev/cxgbe/tom/t4_tom.c
--- a/sys/dev/cxgbe/tom/t4_tom.c	Thu Jul 31 22:08:32 2014 -0700
+++ b/sys/dev/cxgbe/tom/t4_tom.c	Thu Jul 31 22:18:16 2014 -0700
@@ -30,6 +30,7 @@
 
 #include "opt_inet.h"
 #include "opt_inet6.h"
+#include "opt_kdtrace.h"
 
 #include <sys/param.h>
 #include <sys/types.h>
@@ -39,6 +40,7 @@
 #include <sys/module.h>
 #include <sys/protosw.h>
 #include <sys/domain.h>
+#include <sys/sdt.h>
 #include <sys/socket.h>
 #include <sys/socketvar.h>
 #include <sys/taskqueue.h>
@@ -64,6 +66,8 @@
 #include "tom/t4_tom_l2t.h"
 #include "tom/t4_tom.h"
 
+SDT_PROVIDER_DEFINE(t4_tom);
+
 static struct protosw ddp_protosw;
 static struct pr_usrreqs ddp_usrreqs;
 
diff -r 3b3adad9a731 sys/modules/cxgbe/if_cxgbe/Makefile
--- a/sys/modules/cxgbe/if_cxgbe/Makefile	Thu Jul 31 22:08:32 2014 -0700
+++ b/sys/modules/cxgbe/if_cxgbe/Makefile	Thu Jul 31 22:18:16 2014 -0700
@@ -12,6 +12,7 @@ SRCS=	bus_if.h
 SRCS+=	device_if.h
 SRCS+=	opt_inet.h
 SRCS+=	opt_inet6.h
+SRCS+=	opt_kdtrace.h
 SRCS+=	opt_ofed.h
 SRCS+=	pci_if.h
 SRCS+=	t4_hw.c
diff -r 3b3adad9a731 sys/modules/cxgbe/tom/Makefile
--- a/sys/modules/cxgbe/tom/Makefile	Thu Jul 31 22:08:32 2014 -0700
+++ b/sys/modules/cxgbe/tom/Makefile	Thu Jul 31 22:18:16 2014 -0700
@@ -12,6 +12,7 @@ SRCS=	bus_if.h
 SRCS+=	device_if.h
 SRCS+=	opt_inet.h
 SRCS+=	opt_inet6.h
+SRCS+=	opt_kdtrace.h
 SRCS+=	pci_if.h
 SRCS+=	t4_connect.c
 SRCS+=	t4_cpl_io.c
